<style>
    .cabinet-display {
        font-style: italic;
    }

    .label-cabinet a {
        color: black;
        text-decoration: none !important;
        white-space: normal;
    }

    #jstree {
        max-width: 100%;
        overflow: auto;
        font: 11px Verdana, sans-serif;
        padding: 0px;
        padding-bottom: 10px; /* Padding for scrollbar */
    }
</style>
