from mayan.apps.appearance.classes import Icon

icon_document_version_ocr_content_view = Icon(driver_name='fontawesome', symbol='font')
icon_document_version_ocr_content_delete = Icon(
    driver_name='fontawesome-dual', primary_symbol='font',
    secondary_symbol='times'
)
icon_document_version_page_ocr_content_view = Icon(driver_name='fontawesome', symbol='font')
icon_document_version_ocr_download = Icon(
    driver_name='fontawesome-dual', primary_symbol='font',
    secondary_symbol='arrow-down'
)
icon_document_version_ocr_errors_list = Icon(
    driver_name='fontawesome-dual', primary_symbol='font',
    secondary_symbol='exclamation'
)
icon_document_type_ocr_settings = Icon(
    driver_name='fontawesome', symbol='font'
)
icon_document_type_submit = Icon(
    driver_name='fontawesome-dual', primary_symbol='font',
    secondary_symbol='arrow-right'
)
icon_document_version_multiple_ocr_submit = Icon(
    driver_name='fontawesome-dual', primary_symbol='font',
    secondary_symbol='arrow-right'
)
icon_entry_list = Icon(
    driver_name='fontawesome-dual', primary_symbol='font',
    secondary_symbol='exclamation'
)
icon_document_version_ocr_submit = icon_document_version_multiple_ocr_submit
