BINARY_DOCKER_COMPOSE_VERSION = '1.29.1'
DEFAULT_DIRECTORY_BASE = '/opt/'
DEFAULT_DIRECTORY_INSTALLATION = '/opt/mayan-edms/'
DEFAULT_DIRECTORY_MEDIA_ROOT = '/opt/mayan-edms/media/'
DEFAULT_DATABASE_NAME = 'mayan'
DEFAULT_DATABASE_PASSWORD = 'mayanuserpass'
DEFAULT_DATABASE_USER = 'mayan'
DEFAULT_OS_GROUP = 'mayan'
DEFAULT_OS_USERNAME = 'mayan'
DEFAULT_RABBITMQ_PASSWORD = 'mayanrabbitmqpassword'
DEFAULT_RABBITMQ_USER = 'mayan'
DEFAULT_RABBITMQ_VHOST = 'mayan'
DEFAULT_REDIS_PASSWORD = 'mayanredispassword'
DEFAULT_SECRET_KEY = 'secret-key-missing!'
DEFAULT_USER_SETTINGS_FOLDER = 'user_settings'
DOCKER_HOST_REGISTRY_NAME = 'docker-registry.local'
DOCKER_HOST_REGISTRY_PORT = 5000
DOCKER_IMAGE_MAYAN_NAME = 'mayanedms/mayanedms'
DOCKER_LINUX_IMAGE_VERSION = 'ubuntu:20.04'
DOCKER_MYSQL_IMAGE_VERSION = 'mysql:8.0'
DOCKER_ORACLE_IMAGE_VERSION = 'wnameless/oracle-xe-11g-r2'
DOCKER_POSTGRES_IMAGE_VERSION = 'postgres:10.15-alpine'
DOCKER_PYTHON_IMAGE_VERSION = 'python:3.8-slim'
DOCKER_RABBITMQ_IMAGE_VERSION = 'rabbitmq:3.8-alpine'
DOCKER_REDIS_IMAGE_VERSION = 'redis:6.0-alpine'
GUNICORN_JITTER = 50
GUNICORN_MAX_REQUESTS = 500
GUNICORN_TIMEOUT = 120
GUNICORN_WORKER_CLASS = 'sync'
GUNICORN_WORKERS = 3
PYTHON_AMQP_VERSION = '2.6.1'
PYTHON_FLOWER_VERSION = '0.9.7'
PYTHON_MYSQL_VERSION = '2.0.3'
PYTHON_ORACLE_VERSION = '8.1.0'
PYTHON_PIP_VERSION = '21.0.1'
PYTHON_PSYCOPG2_VERSION = '2.8.6'
PYTHON_PSUTIL_VERSION = '5.7.2'
PYTHON_REDIS_VERSION = '3.5.3'
SOURCE_CODE_REPOSITORY = 'https://gitlab.com/mayan-edms/mayan-edms/'
SOURCE_CODE_GIT = 'https://gitlab.com/mayan-edms/mayan-edms.git'
SOURCE_CODE_ISSUES = 'https://gitlab.com/mayan-edms/mayan-edms/issues/'
SECRET_KEY_FILENAME = 'SECRET_KEY'
SYSTEM_DIR = 'system'
SUPERVISOR_CONFIGURATION_DIRECTORY = '/etc/supervisor/conf.d/'
SUPERVISOR_CONFIGURATION_FILENAME = 'mayan-edms.conf'
