__title__ = 'Mayan EDMS'
__version__ = '4.0.3'
__build__ = 0x040003
__build_string__ = 'v4.0.3_Thu Jun 3 06:00:42 2021 -0400'
__django_version__ = '2.2'
__author__ = 'Roberto Rosario'
__author_email__ = 'roberto.rosario@mayan-edms.com'
__description__ = 'Free Open Source Electronic Document Management System'
__license__ = 'Apache 2.0'
__copyright_short__ = '2011 Roberto Rosario'
__copyright__ = '{} {}'.format('Copyright', __copyright_short__)
__website__ = 'https://www.mayan-edms.com'
