TEST_QUOTA_DOTTED_PATH = 'mayan.apps.quotas.tests.quota_backends.TestQuota'
TEST_QUOTA_WITH_MIXINS_DOTTED_PATH = 'mayan.apps.quotas.tests.quota_backends.TestQuotaWithMixins'
TEST_QUOTA_TEST_LIMIT = 1
TEST_QUOTA_TEST_LIMIT_EDITED = 2
TEST_QUOTA_DATA = {'test_limit': TEST_QUOTA_TEST_LIMIT}
